import 'dart:io' show Platform;

import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';

class Intro extends StatelessWidget {
  final VoidCallback onSkip;
  final VoidCallback onProceed;

  Intro({Key? key, required this.onSkip, required this.onProceed})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    final bool isMobile = !kIsWeb && (Platform.isAndroid || Platform.isIOS);

    return Container(
      width: 350,
      margin: EdgeInsets.all(8),
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Align(
            alignment: Alignment.topRight,
            child: IconButton(
              icon: Icon(Icons.close),
              iconSize: 20,
              color: Colors.black54,
              onPressed: onSkip,
            ),
          ),
          Container(
            margin: EdgeInsets.symmetric(vertical: 8, horizontal: 16),
            child: Image(
              image: AssetImage(
                'packages/localizely_sdk/assets/images/localizely-logo.png',
              ),
            ),
          ),
          Container(
            margin: EdgeInsets.symmetric(vertical: 8, horizontal: 16),
            child: Text(
              'In-Context Editing',
              textAlign: TextAlign.center,
              style: Theme.of(context).textTheme.headline1,
            ),
          ),
          Container(
              margin: EdgeInsets.symmetric(vertical: 8, horizontal: 16),
              child: Text(
                isMobile
                    ? 'Scan the QR code from Localizely and start translating the texts in the app'
                    : 'Enter the token from Localizely and start translating the texts in the app',
                textAlign: TextAlign.center,
                style: Theme.of(context).textTheme.bodyText2,
              )),
          Container(
            margin: EdgeInsets.only(top: 50),
            child: ElevatedButton(
              child: Text(
                isMobile ? 'Scan QR Code' : 'Enter token',
              ),
              onPressed: onProceed,
            ),
          ),
        ],
      ),
    );
  }
}
